/*
 * mql_execution_environment.cpp
 *
 * MQL execution environment
 *
 * Ulrik Petersen
 * Created: 5/1-2001 (1st of May, 2001)
 * Last update: 3/21-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <mql_execution_environment.h>

#undef yyFlexLexer
#define yyFlexLexer xxFlexLexer
#include <MQLFlexLexer.h>


CMQL_execution_environment::CMQL_execution_environment(EMdFDB* pMyDB, CEMdFOutput *pMyOut) : m_object_type_names(true)
{
  pLexer = 0;
  pStatement = 0;
  pSymbolTable = 0;
  pDB = pMyDB;
  pOut = pMyOut;
  pError = new CMQLError();
  pEmptySheaf = new CSheaf(false);
  nCompilerStage = COMPILER_STAGE_NONE;
  bQuit = false;
  m_nProgressCount = 0;
  fillOTNs();
}

CMQL_execution_environment::~CMQL_execution_environment()
{
  clean();
  delete pError;
  delete pStatement;
  delete pSymbolTable;
  delete pLexer;
  delete pEmptySheaf;
}

std::string CMQL_execution_environment::get_object_type_name_from_index(short int index)
{
  return *(m_object_type_names[index]);
}

void CMQL_execution_environment::clean(void)
{
  delete pLexer; 
  pLexer = 0;
  delete pStatement;
  pStatement = 0;
  delete pSymbolTable;
  pSymbolTable = 0;
  nCompilerStage = COMPILER_STAGE_NONE;
  bQuit = false;
  m_nProgressCount = 0;
}



short int CMQL_execution_environment::addObjectTypeName(std::string object_type_name)
{
  std::string *pOTN = new std::string(NormalizeOTName(object_type_name));
  bool bWasAdded;
  short int iResult;
  iResult = m_object_type_names.add_value(pOTN, strptr_equal(), bWasAdded);
  if (!bWasAdded) {
    delete pOTN;
  }
  return iResult;
}

// Fill m_object_type_names with object types from DB
void CMQL_execution_environment::fillOTNs()
{
  // Return if pDB hasn't been initialized
  if (!pDB) {
    return;
  }

  // Return if connection isn't OK
  if (!pDB->ConnectionOk()) {
    return;
  }

  // Check that table object_types exists in DB
  bool bIsInitialized;
  if (!pDB->DBIsInitialized(bIsInitialized)) {
    return;
  }
  if (!bIsInitialized) {
    return;
  }

  // Get list of object types in database
  std::list<std::string> object_types;
  if (!pDB->GetObjectTypes(object_types)) {
    return;
  }

  // Add object types to m_object_type_names;
  std::list<std::string>::const_iterator ci(object_types.begin());
  std::list<std::string>::const_iterator cend(object_types.end());
  while (ci != cend) {
    addObjectTypeName(*ci);
    ++ci;
  }
}
