/*
 * mql_mo_nr.cpp
 *
 * MQL mo_nr
 *
 * Ulrik Petersen
 * Created: 3/16-2001
 * Last update: 1/18-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/



#include <mql_mo_nr.h>
#include <mql_sheaf.h>
#include <assert.h>




///////////////////////////////////////////////////////////////
//
// CMo_nr
//
//////////////////////////////////////////////////////////////

CMo_nr::CMo_nr(CMatched_object* pMo, bool nr) : m_pMo(pMo), m_nr(nr)
{
}

CMo_nr::CMo_nr(const CMo_nr& other)
{
  copy_other(other);
}

CMo_nr::~CMo_nr()
{
  delete m_pMo;
}

CMatched_object* CMo_nr::get_copy_of_mo() const
{
  CMatched_object* pMo;
  pMo = new CMatched_object(*m_pMo);
  return pMo;
}

bool CMo_nr::get_nr() const
{
  return m_nr;
}

CMo_nr& CMo_nr::operator=(const CMo_nr& other)
{
  delete m_pMo;
  copy_other(other);
  return *this;
}

void CMo_nr::copy_other(const CMo_nr& other)
{
  m_pMo = new CMatched_object(*other.m_pMo);
  m_nr = other.m_nr;
}

bool CMo_nr::is_NIL_mo(void) const
{
  if (m_pMo == 0)
    return false;
  else 
    return m_pMo->get_kind() == kMOKNIL_mo;
}

bool CMo_nr::is_EMPTY_mo(void) const
{
  if (m_pMo == 0)
    return false;
  else 
    return m_pMo->get_kind() == kMOKEMPTY_mo;
}

monad_m CMo_nr::get_last(void)
{
  assert(m_pMo != 0);
  return m_pMo->get_last();
}

const CMatched_object& CMo_nr::get_const_reference() const
{
  assert(m_pMo != 0);
  return *m_pMo;
}

bool CMo_nr::operator==(const CMo_nr& other) const
{
  return m_pMo == other.m_pMo && m_nr == other.m_nr;
}

// This is not a meaningful operation on CMo_nr
bool CMo_nr::operator<(const CMo_nr& other) const 
{
  return false;
}


///////////////////////////////////////////////////////////////
//
// CList_of_mo_nr
//
//////////////////////////////////////////////////////////////

CList_of_mo_nr::CList_of_mo_nr() : m_list()
{
}

CList_of_mo_nr::~CList_of_mo_nr() 
{
}

void CList_of_mo_nr::append(const CMo_nr& mo_nr) 
{ 
  m_list.push_back(mo_nr); 
}

CList_of_mo_nr::const_iterator CList_of_mo_nr::begin() const 
{ 
  return m_list.begin(); 
}

CList_of_mo_nr::const_iterator CList_of_mo_nr::end() const 
{ 
  return m_list.end(); 
}

bool CList_of_mo_nr::has_NIL_mo(void) const 
{ 
  return begin()->is_NIL_mo(); 
}

bool CList_of_mo_nr::has_EMPTY_mo(void) const 
{ 
  return begin()->is_EMPTY_mo(); 
}

