/*
 * mql_sheaf.cpp
 *
 * MQL sheafs
 *
 * Ulrik Petersen
 * Created: 3/8-2001
 * Last update: 6/24-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <mql_sheaf.h>
#include <mql_execution_environment.h>
#include <mql_utility.h>
#include <string_func.h>
#include <cassert>

/////////////////////////////////////////////////////////////////
//
// StrawConstIterator
//
/////////////////////////////////////////////////////////////////
StrawConstIterator::StrawConstIterator(const CStraw *pMotherStraw)
  : m_pMotherStraw(pMotherStraw)
{
  m_iter = m_pMotherStraw->m_list.begin();
}

StrawConstIterator::StrawConstIterator()
  :  m_pMotherStraw(0)
{
}

StrawConstIterator::StrawConstIterator(const StrawConstIterator& other)
{
  assign(other);
}


StrawConstIterator::~StrawConstIterator()
{
}

StrawConstIterator& StrawConstIterator::operator=(const StrawConstIterator& other)
{
  assign(other);
  return *this;
}

void StrawConstIterator::assign(const StrawConstIterator& other)
{
  m_pMotherStraw = other.m_pMotherStraw;
  m_iter = other.m_iter;
}


bool StrawConstIterator::hasNext() const
{
  assert(m_pMotherStraw != 0);
  return m_iter != m_pMotherStraw->m_list.end();
}


const CMatched_object* StrawConstIterator::next()
{
  assert(m_pMotherStraw != 0);
  assert(hasNext());
  CMatched_object *pMO = *m_iter;
  ++m_iter;
  return pMO;
}

const CMatched_object* StrawConstIterator::previous()
{
  assert(m_pMotherStraw != 0);
  --m_iter;
  return *m_iter;
}

const CMatched_object* StrawConstIterator::current()
{
  assert(m_pMotherStraw != 0);
  return *m_iter;
}





/////////////////////////////////////////////////////////////////
//
// SheafIterator
//
/////////////////////////////////////////////////////////////////
SheafIterator::SheafIterator(CList_of_straws *pMotherList)
  : m_pMotherList(pMotherList)
{
  m_iter = m_pMotherList->m_list.begin();
}

SheafIterator::SheafIterator()
  :  m_pMotherList(0)
{
}

SheafIterator::SheafIterator(const SheafIterator& other)
{
  assign(other);
}


SheafIterator::~SheafIterator()
{
}

SheafIterator& SheafIterator::operator=(const SheafIterator& other)
{
  assign(other);
  return *this;
}

void SheafIterator::assign(const SheafIterator& other)
{
  m_pMotherList = other.m_pMotherList;
  m_iter = other.m_iter;
}


bool SheafIterator::hasNext() const
{
  assert(m_pMotherList != 0);
  return m_iter != m_pMotherList->m_list.end();
}


CStraw* SheafIterator::next()
{
  assert(m_pMotherList != 0);
  assert(hasNext());
  CStraw *pStraw = *m_iter;
  ++m_iter;
  return pStraw;
}

CStraw* SheafIterator::previous()
{
  assert(m_pMotherList != 0);
  --m_iter;
  return *m_iter;
}

CStraw* SheafIterator::current()
{
  assert(m_pMotherList != 0);
  return *m_iter;
}



/////////////////////////////////////////////////////////////////
//
// SheafConstIterator
//
/////////////////////////////////////////////////////////////////
SheafConstIterator::SheafConstIterator(const CList_of_straws *pMotherList)
  : m_pMotherList(pMotherList)
{
  m_iter = m_pMotherList->m_list.begin();
}

SheafConstIterator::SheafConstIterator()
  :  m_pMotherList(0)
{
}

SheafConstIterator::SheafConstIterator(const SheafConstIterator& other)
{
  assign(other);
}


SheafConstIterator::~SheafConstIterator()
{
}

SheafConstIterator& SheafConstIterator::operator=(const SheafConstIterator& other)
{
  assign(other);
  return *this;
}

void SheafConstIterator::assign(const SheafConstIterator& other)
{
  m_pMotherList = other.m_pMotherList;
  m_iter = other.m_iter;
}


bool SheafConstIterator::hasNext() const
{
  assert(m_pMotherList != 0);
  return m_iter != m_pMotherList->m_list.end();
}


const CStraw* SheafConstIterator::next()
{
  assert(m_pMotherList != 0);
  assert(hasNext());
  CStraw *pStraw = *m_iter;
  ++m_iter;
  return pStraw;
}

const CStraw* SheafConstIterator::previous()
{
  assert(m_pMotherList != 0);
  --m_iter;
  return *m_iter;
}

const CStraw* SheafConstIterator::current()
{
  assert(m_pMotherList != 0);
  return *m_iter;
}



/////////////////////////////////////////////////////////////////
//
// CMatched_object
//
/////////////////////////////////////////////////////////////////
CMatched_object::CMatched_object(eMOKind kind) // for kMOKNIL_mo, kMOKEMPTY_mo
{
  assert(kind == kMOKNIL_mo || kind == kMOKEMPTY_mo);
  m_kind = kind;
  m_id_d = NIL;
  if (kind == kMOKNIL_mo)
    m_object_type_index = 0;
  else
    m_object_type_index = -1; // kMOKEMPTY_mo
  m_sheaf = 0;
  m_focus = false;
  m_last = 0;
}

CMatched_object::CMatched_object(id_d_t id_d, short int object_type_index, const CSet_of_monad_ms& monads, bool focus, CSheaf* sheaf) : m_monads() // for kMOKID_D
{
  m_kind = kMOKID_D;
  m_id_d = id_d;
  m_object_type_index = object_type_index;
  m_sheaf = sheaf;
  m_focus = focus;
  m_monads = monads;
  m_last = m_monads.last();
}

CMatched_object::CMatched_object(const CSet_of_monad_ms& monads, bool focus, CSheaf* sheaf) : m_monads(monads) // for kMOKID_M
{
  m_kind = kMOKID_M;
  m_id_d = NIL;
  m_object_type_index = -1;
  m_sheaf = sheaf;
  m_focus = focus;
  m_last = monads.last();
}

CMatched_object::CMatched_object(const CMatched_object& other)
  :  m_monads(other.m_monads),
     m_id_d(other.m_id_d),
     m_kind(other.m_kind),
     m_focus(other.m_focus),
     m_object_type_index(other.m_object_type_index),
     m_last(other.m_last)
{
  if (other.m_sheaf == 0) {
    m_sheaf = 0;
  } else {
    m_sheaf = new CSheaf(*other.m_sheaf);
  }
}

CMatched_object::~CMatched_object()
{
  delete m_sheaf;
}

const CSet_of_monad_ms& CMatched_object::get_monads(void) const
{
  assert(m_kind == kMOKID_M || m_kind == kMOKID_D);
  return m_monads;
}

id_d_t CMatched_object::get_id_d(void) const
{
  assert(m_kind == kMOKID_D);
  return m_id_d;
}

const CSheaf* CMatched_object::get_sheaf(void) const
{
  assert(m_kind == kMOKID_D || m_kind == kMOKID_M);
  return m_sheaf;
}

std::string CMatched_object::getObjectTypeName(CMQL_execution_environment *pEE) const
{
  return pEE->get_object_type_name_from_index(m_object_type_index);
}

CMatched_object& CMatched_object::operator=(const CMatched_object& other)
{
  delete m_sheaf;
  copy_other(other);
  return *this;
}

void CMatched_object::copy_other(const CMatched_object& other)
{
  m_kind = other.m_kind;
  m_monads = other.m_monads;
  m_id_d = other.m_id_d;
  m_object_type_index = other.m_object_type_index;
  if (other.m_sheaf == 0) {
    m_sheaf = 0;
  } else {
    m_sheaf = new CSheaf(*other.m_sheaf);
  }
  m_focus = other.m_focus;
  m_last = other.m_last;
}

void CMatched_object::put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const
{
  std::string object_type_name;
  std::string monads_string = m_monads.ToString();
  
  switch (m_kind) {
  case kMOKID_D:
    // Get object type name
    object_type_name = pEE->get_object_type_name_from_index(m_object_type_index);

    // Print matched_object
    pOut->out("[ " + object_type_name + " " + id_d2string(m_id_d) 
	      + monads_string + bool_alpha2string(m_focus) + " ");
    if (!sheaf_is_empty()) {
      m_sheaf->put_Console(pOut, pEE);
    } else {
      pEE->pEmptySheaf->put_Console(pOut, pEE);
    }
    pOut->out(" ]\n");
    break;
  case kMOKID_M:
    pOut->out("[ pow_m" 
	      + monads_string + bool_alpha2string(m_focus) + " ");
    if (!sheaf_is_empty()) {
      m_sheaf->put_Console(pOut, pEE);
    } else {
      pEE->pEmptySheaf->put_Console(pOut, pEE);
    }
    pOut->out(" ]\n");
    break;
  case kMOKNIL_mo:
  case kMOKEMPTY_mo:
  default:
    assert(false); // cannot output NIL_mo or EMPTY_mo
  };
}

void CMatched_object::put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const
{
  std::string object_type_name;
  AttributePairList attribute_list;
  switch (m_kind) {
  case kMOKID_D:
    // Get object type name
    object_type_name = pEE->get_object_type_name_from_index(m_object_type_index);

    // Make attribute list
    attribute_list.push_back(AttributePair("object_type_name", object_type_name));
    attribute_list.push_back(AttributePair("focus", bool_alpha2string(m_focus)));
    attribute_list.push_back(AttributePair("id_d", id_d2string(m_id_d)));

    // Print matched_object
    pOut->start_tag("matched_object", attribute_list, true);
    m_monads.PrintXML(pOut);
    if (!sheaf_is_empty()) {
      m_sheaf->put_XML(pOut, pEE);
    } else {
      pEE->pEmptySheaf->put_XML(pOut, pEE);
    }
    pOut->end_tag("matched_object", true);
    break;
  case kMOKID_M:
    // Make attribute list
    attribute_list.push_back(AttributePair("object_type_name", "pow_m"));
    attribute_list.push_back(AttributePair("focus", bool_alpha2string(m_focus)));
    attribute_list.push_back(AttributePair("id_d", NIL_AS_VISIBLE_STRING));

    // Print matched_object
    pOut->start_tag("matched_object", attribute_list, true);
    m_monads.PrintXML(pOut);
    if (!sheaf_is_empty()) {
      m_sheaf->put_XML(pOut, pEE);
    } else {
      pEE->pEmptySheaf->put_XML(pOut, pEE);
    }
    pOut->end_tag("matched_object", true);
    break;
  case kMOKNIL_mo:
  case kMOKEMPTY_mo:
  default:
    assert(false); // cannot output NIL_mo or EMPTY_mo
  };
}


/////////////////////////////////////////////////////////////////
//
// CStraw
//
/////////////////////////////////////////////////////////////////
CStraw::CStraw() : m_last(0)
{
}

CStraw::CStraw(const CStraw& other) : m_last(0)
{
  copy_other(other);
}

CStraw::~CStraw()
{
  delete_mos();
}

StrawConstIterator CStraw::const_iterator() const
{
  return StrawConstIterator(this);
}

void CStraw::copy_other(const CStraw& other)
{
  m_last = other.m_last;
  StrawConstIterator oi = other.const_iterator();
  while (oi.hasNext()) {
    m_list.push_back(new CMatched_object(*(oi.current())));
    oi.next();
  }
}

void CStraw::delete_mos(void)
{
  MOList::iterator i;
  i = m_list.begin();
  while (i != m_list.end()) {
    delete *i;
    ++i;
  }
  m_list.erase(m_list.begin(), m_list.end());
}

void CStraw::append(CMatched_object* mo)
{
  m_list.push_back(mo);
  update_last(mo);
}

void CStraw::update_last(const CMatched_object* mo) 
{
  monad_m mo_last = mo->m_last;
  if (mo_last > m_last)
    m_last = mo_last;
}


void CStraw::put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const
{
  pOut->out(" < ");
  StrawConstIterator ci = const_iterator();
  while (ci.hasNext()) {
    // Output matched_object
    (ci.current())->put_Console(pOut, pEE);
    ci.next();
  }
  pOut->out(" > \n");
}

void CStraw::put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const
{
  pOut->start_tag("straw", true);
  StrawConstIterator ci = const_iterator();
  while (ci.hasNext()) {
    // Output matched_object
    (ci.current())->put_XML(pOut, pEE);
    ci.next();
  }
  pOut->end_tag("straw", true);
}

CStraw& CStraw::operator=(const CStraw& other)
{
  delete_mos();
  copy_other(other);
  return *this;
}

void CStraw::prepend(const CStraw& other)
{
  MOList::const_reverse_iterator ci;
  MOList::const_reverse_iterator crend(other.m_list.rend());
  ci = other.m_list.rbegin();
  while (ci != crend) {
    const CMatched_object *pOtherObj = *ci;
    m_list.insert(m_list.begin(), new CMatched_object(*pOtherObj));
    update_last(pOtherObj);
    ++ci;
  }
}


/////////////////////////////////////////////////////////////////
//
// CList_of_straws
//
/////////////////////////////////////////////////////////////////
CList_of_straws::CList_of_straws() 
{
}

CList_of_straws::CList_of_straws(const CList_of_straws& other)
{
  copy_other(other);
}

CList_of_straws::~CList_of_straws()
{
  delete_straws();
}

SheafIterator CList_of_straws::iterator() 
{
  return SheafIterator(this);
}

SheafConstIterator CList_of_straws::const_iterator() const
{
  return SheafConstIterator(this);
}

void CList_of_straws::append(CStraw* straw)
{
  m_list.push_back(straw);
}

void CList_of_straws::append_and_subsume(CList_of_straws* pOther)
{
  SheafIterator ci = pOther->iterator();
  while (ci.hasNext()) {
    append(ci.current());
    *ci.m_iter = 0;
    ci.next();
  }
  delete pOther;
}





void CList_of_straws::put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const
{
  pOut->out(" < ");
  SheafConstIterator ci = const_iterator();
  while (ci.hasNext()) {
    // Output straw
    (ci.current())->put_Console(pOut, pEE);
    ci.next();
  }
  pOut->out(" > \n");
}

void CList_of_straws::put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const
{
  SheafConstIterator ci = const_iterator();
  while (ci.hasNext()) {
    // Output straw
    (ci.current())->put_XML(pOut, pEE);
    ci.next();
  }
}

CList_of_straws& CList_of_straws::operator=(const CList_of_straws& other)
{
  delete_straws();
  copy_other(other);
  return *this;
}

void CList_of_straws::copy_other(const CList_of_straws& other)
{
  SheafConstIterator oi = other.const_iterator();
  while (oi.hasNext()) {
    m_list.push_back(new CStraw(*(oi.current())));
    oi.next();
  }
}

void CList_of_straws::delete_straws(void)
{
  StrawList::iterator i;
  i = m_list.begin();
  while (i != m_list.end()) {
    delete *i;
    ++i;
  }
  m_list.erase(m_list.begin(), m_list.end());
}

//
// Makes a copy of s
void CList_of_straws::Join(CStraw* s)
{
  if (m_list.empty()) {
    CStraw* pStraw = new CStraw(*s);
    m_list.push_back(pStraw);
  } else {
    StrawList::iterator i(m_list.begin());
    StrawList::iterator e(m_list.end());
    while (i != e) {
      (*i)->prepend(*s);
      ++i;
    }
  }
}



/////////////////////////////////////////////////////////////////
//
// CSheaf
//
/////////////////////////////////////////////////////////////////
// fail
CSheaf::CSheaf(bool is_fail)
{
  if (is_fail) {
    m_plist = 0;
  } else {
    m_plist = new CList_of_straws();
  }
}

CSheaf::CSheaf(CList_of_straws* plist)
{
  m_plist = plist;
}

CSheaf::CSheaf(const CSheaf& other)
{
  copy_other(other);
}

CSheaf::~CSheaf()
{
  delete m_plist;
}

SheafIterator CSheaf::iterator()
{
  return m_plist->iterator();
}

SheafConstIterator CSheaf::const_iterator() const
{
  return m_plist->const_iterator();
}

void CSheaf::copy_other(const CSheaf& other)
{
  m_plist = new CList_of_straws(*other.m_plist);
}

void CSheaf::delete_straws(void)
{
  delete m_plist;
}

bool CSheaf::is_fail(void) const
{
  return m_plist == 0;
}

CSheaf& CSheaf::operator=(const CSheaf& other)
{
  delete_straws();
  copy_other(other);
  return *this;
}

void CSheaf::put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const
{
  if (is_fail()) {
    pOut->out(" // \n");
  } else {
    pOut->out(" // ");
    m_plist->put_Console(pOut, pEE);
  }
}

void CSheaf::put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const
{
  if (is_fail()) {
    // Do nothing
  } else {
    pOut->start_tag("sheaf", true);
    m_plist->put_XML(pOut, pEE);
    pOut->end_tag("sheaf", true);
  }
}

void CSheaf::print_DTD(CEMdFOutput* pOut) 
{
  pOut->out("  <!ELEMENT sheaf (straw)* >"); 
  pOut->newline();
  pOut->out("  <!ELEMENT straw (matched_object)* >"); 
  pOut->newline();
  pOut->out("  <!ELEMENT matched_object (monad_set, (sheaf)?) >"); 
  pOut->newline();
  pOut->out("  <!ELEMENT monad_set (mse)+ >"); 
  pOut->newline();
  pOut->out("  <!ELEMENT mse EMPTY>"); 
  pOut->newline();
  pOut->out("  <!ATTLIST sheaf >"); 
  pOut->newline();
  pOut->out("  <!ATTLIST straw >"); 
  pOut->newline();
  pOut->out("  <!ATTLIST matched_object "); 
  pOut->newline();
  pOut->out("            object_type_name CDATA #REQUIRED "); 
  pOut->newline();
  pOut->out("            focus (false | true) #REQUIRED"); 
  pOut->newline();
  pOut->out("            id_d CDATA \"nil\""); 
  pOut->newline();
  pOut->out("  >"); 
  pOut->newline();
  pOut->out("  <!ATTLIST monad_set >"); 
  pOut->newline();
  pOut->out("  <!ATTLIST mse"); 
  pOut->newline();
  pOut->out("            first CDATA #REQUIRED"); 
  pOut->newline();
  pOut->out("            last  CDATA #REQUIRED"); 
  pOut->newline();
  pOut->out("  >"); 
  pOut->newline();
}



