/*
 * emdf.h
 *
 * Basic definitions for EMdF
 *
 * Ulrik Petersen
 * Created: 1/27-2001
 * Last update: 7/6-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef EMDF__H__
#define EMDF__H__

#include <emdros-lconfig.h>
#include <emdros-version.h>

typedef long id_d_t;
typedef long monad_m;

#define SEQUENCE_OBJECT_ID_DS   (0)
#define SEQUENCE_OTHER_IDS      (1)
#define SEQUENCE_START          (10001)

#define MIN_ENUM_TYPE_ID        SEQUENCE_START


#define MAX_DBNAME_CHARS         (32)
#define MAX_IDENTIFIER_CHARS     (255)

#define MAX_MONAD               (2100000000L)

/* If:
 *   - SEQUENCE_START is 10001, and 
 *   - the highest FEATURE_TYPE_X used for strings is 3, and

 *   - the length is multiplied by 10 and added to the FEATURE_TYPE_X
 *     in order to encode feature type, and
 *   - we need double the space for STRING strings (since we encode
 *     them using hexadecimal notation), 
 *
 * Then MAX_STRING_LENGTH can be at most 
 *                  ((10001 - 3)/10)/2 = 499 
 * However, with MySQL, the longest string length is 255.  Therefore,
 * MAX_STRING_LENGTH must be 255/2 = 127
 */

#if USE_POSTGRESQL
#define MAX_STRING_LENGTH      (499)
#elif USE_MYSQL
#define MAX_STRING_LENGTH      (127)
#elif SE_ODBC
#define MAX_STRING_LENGTH      (127)
#endif



#define FEATURE_TYPE_INTEGER    (0)
#define FEATURE_TYPE_STRING     (1)
#define FEATURE_TYPE_ASCII      (2)
#define FEATURE_TYPE_ID_D       (3)

#define NIL                     (0)
#define NIL_AS_STRING           "0"
#define NIL_AS_VISIBLE_STRING   "nil"

#define OBJECT_TYPE_POW_M       (1)


/*
 * Emdros logging
 */
#ifdef WIN32
#define EMDROS_LOGGING     (0)
#else
#define EMDROS_LOGGING     (0)
#define EMDROS_LOGFILE     "/tmp/emdros.log"
#endif


/*
 * Emdros versions 
 */

#define EMDROS_VERSION_PRE_1_1_0       (0)
#define EMDROS_VERSION_1_1_0_TO_1_1_4  (1)
#define EMDROS_VERSION_1_1_5_TO_1_1_7  (2)
#define EMDROS_VERSION_1_1_8           (3)

#endif /* EMDF__H__ */
