/*
 * inst.h
 *
 * inst(T,Su) class
 *
 * Ulrik Petersen
 * Created: 2/26-2001
 * Last update: 2/1-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/



#ifndef INST__H__
#define INST__H__

#include <emdf.h>
#include <inst_object.h>
#include <monads.h>
#include <list>
#include <vector>
#include <map>
#include <string>

typedef std::list<id_d_t> IDDListList;

class IDDList {
private:
  IDDListList m_list;
public:
  typedef IDDListList::const_iterator const_iterator;
  IDDList();
  ~IDDList();
  void AddOid(id_d_t oid) { m_list.push_back(oid); };
  IDDList::const_iterator begin() const;
  IDDList::const_iterator end() const;
  long length() const;
  void make_empty();
private:
  /* Make the copy-constructor and assignment operator
   * private so that they are not used. */
  IDDList(const IDDList& other);
  IDDList& operator=(const IDDList& other);
};

typedef std::multimap<monad_m, CInstObject*> Monad_m2InstObjectMap;

class CInstContents {
private:
  std::vector<std::string> m_feature_names;
  Monad_m2InstObjectMap m_map_by_monad_m;
  bool m_is_aggregate;
  monad_m m_min_monad, m_max_monad;
  friend class CInst;
private:
  typedef Monad_m2InstObjectMap::const_iterator const_iterator; 
  CInstContents(const std::vector<std::string>& feature_names);
  ~CInstContents();
  void add_object(CInstObject *pObj);
  void reassign_object(CInstObject *pObj, monad_m previous_first_monad);
  void objects_at_Sm(monad_m Sm, CInstContents::const_iterator& ci1, CInstContents::const_iterator& ci2);
  void big_union(CSet_of_monad_ms& result);
  void set_is_aggregate(bool is_aggregate);
  bool is_aggregate(void) const;
};

class CInst {
private:
  CSet_of_monad_ms m_current_universe;
  monad_m m_first_monad, m_last_monad;
  CInstContents *m_pContents;
  bool m_bDoDelete;
public:
  typedef CInstContents::const_iterator const_iterator; 
  CInst(CInst* pOther);
  CInst(const std::vector<std::string>& feature_names);
  ~CInst();
  CInst::const_iterator next(CInst::const_iterator& ci); // relative to the current universe
  CInst::const_iterator begin() const; // relative to the current universe
  CInst::const_iterator end() const { return m_pContents->m_map_by_monad_m.end(); }; // end of m_map_by_monad_m
  void add_object(CInstObject *pObj) { m_pContents->add_object(pObj); };
  void set_current_universe(const CSet_of_monad_ms& universe) { m_current_universe = universe; m_first_monad = universe.first(); m_last_monad = universe.last(); };
  void objects_at_Sm(monad_m Sm, CInst::const_iterator& ci1, CInst::const_iterator& ci2) { m_pContents->objects_at_Sm(Sm, ci1, ci2); };
  void big_union(CSet_of_monad_ms& result) { m_pContents->big_union(result); };
  void set_is_aggregate(bool is_aggregate) { m_pContents->set_is_aggregate(is_aggregate); };
  bool is_aggregate(void) const { return m_pContents->is_aggregate(); };
};





#endif // INST__H__
