/*
 * mql_execution_environment.h
 *
 * MQL execution environment
 *
 * Ulrik Petersen
 * Created: 5/1-2001 (1st of May, 2001)
 * Last update: 6/21-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/



#ifndef MQL_EXECUTION_ENVIRONMENT__H__
#define MQL_EXECUTION_ENVIRONMENT__H__

#include <emdf_output.h>
#include <mql_types.h>
#include <mql_error.h>
#include <mql_sheaf.h>
#include <emdfdb.h>
#include <monads.h>
#ifndef SWIG
#include <smart_vector.h>
#include <mql_symboltable.h>
#endif // !defined SWIG

#define COMPILER_STAGE_NONE   (0)
#define COMPILER_STAGE_PARSE  (1)
#define COMPILER_STAGE_WEED   (2)
#define COMPILER_STAGE_SYMBOL (3)
#define COMPILER_STAGE_TYPE   (4)
#define COMPILER_STAGE_MONADS (5)
#define COMPILER_STAGE_EXEC   (6)

class xxFlexLexer; // forward declaration

#ifndef SWIG
struct strptr_equal : public std::equal_to<std::string*> {
  bool operator() (const std::string* x, const std::string* y) const { return *x == *y; };
};

typedef CSmartVector<std::string, strptr_equal > StrVec;

#endif // !defined SWIG

class CMQL_execution_environment {
private:
  short m_nProgressCount;
public:
  CStatement    *pStatement;
  CSheaf        *pEmptySheaf;
  EMdFDB* pDB;
  CEMdFOutput *pOut;
  CMQLError *pError;
  CMQL_execution_environment(EMdFDB* pMyDB, CEMdFOutput *pMyOut);
  ~CMQL_execution_environment();
  int nCompilerStage;
  void clean(void);
  std::string get_object_type_name_from_index(short int index);
#ifndef SWIG
  xxFlexLexer *pLexer;
  CMQLSymbolTable* pSymbolTable;
  // all_m_1 for the current MQL query. Set by CQueryBase::MyMonads().
  CSet_of_monad_ms m_all_m_1; 
  bool bQuit;  // Should we quit at next given opportunity?
  StrVec m_object_type_names;
  short int addObjectTypeName(std::string object_type_name);
  void fillOTNs();
  void progress(void); // Called to show progress
#endif // !defined SWIG
};

#ifndef SWIG
inline void CMQL_execution_environment::progress(void)
{
  m_nProgressCount++;
  if ((m_nProgressCount % 100) == 99) {
    if (pOut->get_output_kind() == kOKXML) {
      m_nProgressCount = 0;
      pOut->out(".");
      pOut->flush();
    }
  }
}

#endif // !defined SWIG

#endif // MQL_EXECUTION_ENVIRONMENT__H__
